@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 """
 oauthlib.oauth1.rfc5849
@@ -10,23 +10,32 @@ for signing and checking OAuth 1.0 RFC 5849 requests.
 """
 
 import logging
+import sys
 import time
-import urlparse
+try:
+    import urlparse
+except ImportError:
+    import urllib.parse as urlparse
+
+if sys.version_info[0] == 3:
+    bytes_type = bytes
+else:
+    bytes_type = str
 
 from oauthlib.common import Request, urlencode, generate_nonce
-from oauthlib.common import generate_timestamp
+from oauthlib.common import generate_timestamp, to_unicode
 from . import parameters, signature, utils
 
-SIGNATURE_HMAC = u"HMAC-SHA1"
-SIGNATURE_RSA = u"RSA-SHA1"
-SIGNATURE_PLAINTEXT = u"PLAINTEXT"
+SIGNATURE_HMAC = "HMAC-SHA1"
+SIGNATURE_RSA = "RSA-SHA1"
+SIGNATURE_PLAINTEXT = "PLAINTEXT"
 SIGNATURE_METHODS = (SIGNATURE_HMAC, SIGNATURE_RSA, SIGNATURE_PLAINTEXT)
 
-SIGNATURE_TYPE_AUTH_HEADER = u'AUTH_HEADER'
-SIGNATURE_TYPE_QUERY = u'QUERY'
-SIGNATURE_TYPE_BODY = u'BODY'
+SIGNATURE_TYPE_AUTH_HEADER = 'AUTH_HEADER'
+SIGNATURE_TYPE_QUERY = 'QUERY'
+SIGNATURE_TYPE_BODY = 'BODY'
 
-CONTENT_TYPE_FORM_URLENCODED = u'application/x-www-form-urlencoded'
+CONTENT_TYPE_FORM_URLENCODED = 'application/x-www-form-urlencoded'
 
 
 class Client(object):
@@ -38,16 +47,25 @@ class Client(object):
             callback_uri=None,
             signature_method=SIGNATURE_HMAC,
             signature_type=SIGNATURE_TYPE_AUTH_HEADER,
-            rsa_key=None, verifier=None):
-        self.client_key = client_key
-        self.client_secret = client_secret
-        self.resource_owner_key = resource_owner_key
-        self.resource_owner_secret = resource_owner_secret
-        self.signature_method = signature_method
-        self.signature_type = signature_type
-        self.callback_uri = callback_uri
-        self.rsa_key = rsa_key
-        self.verifier = verifier
+            rsa_key=None, verifier=None, realm=None,
+            encoding='utf-8', nonce=None, timestamp=None):
+
+        # Convert to unicode using encoding if given, else assume unicode
+        encode = lambda x: to_unicode(x, encoding) if encoding else x
+
+        self.client_key = encode(client_key)
+        self.client_secret = encode(client_secret)
+        self.resource_owner_key = encode(resource_owner_key)
+        self.resource_owner_secret = encode(resource_owner_secret)
+        self.signature_method = encode(signature_method)
+        self.signature_type = encode(signature_type)
+        self.callback_uri = encode(callback_uri)
+        self.rsa_key = encode(rsa_key)
+        self.verifier = encode(verifier)
+        self.realm = encode(realm)
+        self.encoding = encode(encoding)
+        self.nonce = encode(nonce)
+        self.timestamp = encode(timestamp)
 
         if self.signature_method == SIGNATURE_RSA and self.rsa_key is None:
             raise ValueError('rsa_key is required when using RSA signature method.')
@@ -93,23 +111,27 @@ class Client(object):
     def get_oauth_params(self):
         """Get the basic OAuth parameters to be used in generating a signature.
         """
+        nonce = (generate_nonce()
+                 if self.nonce is None else self.nonce)
+        timestamp = (generate_timestamp()
+                     if self.timestamp is None else self.timestamp)
         params = [
-            (u'oauth_nonce', generate_nonce()),
-            (u'oauth_timestamp', generate_timestamp()),
-            (u'oauth_version', u'1.0'),
-            (u'oauth_signature_method', self.signature_method),
-            (u'oauth_consumer_key', self.client_key),
+            ('oauth_nonce', nonce),
+            ('oauth_timestamp', timestamp),
+            ('oauth_version', '1.0'),
+            ('oauth_signature_method', self.signature_method),
+            ('oauth_consumer_key', self.client_key),
         ]
         if self.resource_owner_key:
-            params.append((u'oauth_token', self.resource_owner_key))
+            params.append(('oauth_token', self.resource_owner_key))
         if self.callback_uri:
-            params.append((u'oauth_callback', self.callback_uri))
+            params.append(('oauth_callback', self.callback_uri))
         if self.verifier:
-            params.append((u'oauth_verifier', self.verifier))
+            params.append(('oauth_verifier', self.verifier))
 
         return params
 
-    def _render(self, request, formencode=False):
+    def _render(self, request, formencode=False, realm=None):
         """Render a signed request according to signature type
 
         Returns a 3-tuple containing the request URI, headers, and body.
@@ -130,12 +152,12 @@ class Client(object):
         # like the spec requires. This would be a fundamental change though, and
         # I'm not sure how I feel about it.
         if self.signature_type == SIGNATURE_TYPE_AUTH_HEADER:
-            headers = parameters.prepare_headers(request.oauth_params, request.headers)
+            headers = parameters.prepare_headers(request.oauth_params, request.headers, realm=realm)
         elif self.signature_type == SIGNATURE_TYPE_BODY and request.decoded_body is not None:
             body = parameters.prepare_form_encoded_body(request.oauth_params, request.decoded_body)
             if formencode:
                 body = urlencode(body)
-            headers['Content-Type'] = u'application/x-www-form-urlencoded'
+            headers['Content-Type'] = 'application/x-www-form-urlencoded'
         elif self.signature_type == SIGNATURE_TYPE_QUERY:
             uri = parameters.prepare_request_uri_query(request.oauth_params, request.uri)
         else:
@@ -143,7 +165,7 @@ class Client(object):
 
         return uri, headers, body
 
-    def sign(self, uri, http_method=u'GET', body=None, headers=None):
+    def sign(self, uri, http_method='GET', body=None, headers=None, realm=None):
         """Sign a request
 
         Signs an HTTP request with the specified parts.
@@ -168,7 +190,8 @@ class Client(object):
         dicts, for example.
         """
         # normalize request data
-        request = Request(uri, http_method, body, headers)
+        request = Request(uri, http_method, body, headers,
+                          encoding=self.encoding)
 
         # sanity check
         content_type = request.headers.get('Content-Type', None)
@@ -208,10 +231,11 @@ class Client(object):
         request.oauth_params = self.get_oauth_params()
 
         # generate the signature
-        request.oauth_params.append((u'oauth_signature', self.get_oauth_signature(request)))
+        request.oauth_params.append(('oauth_signature', self.get_oauth_signature(request)))
 
         # render the signed request and return it
-        return self._render(request, formencode=True)
+        return self._render(request, formencode=True,
+                            realm=(realm or self.realm))
 
 
 class Server(object):
@@ -454,8 +478,9 @@ class Server(object):
         """Extracts parameters from query, headers and body. Signature type
         is set to the source in which parameters were found.
         """
+        # Per RFC5849, only the Authorization header may contain the 'realm' optional parameter.
         header_params = signature.collect_parameters(headers=request.headers,
-                exclude_oauth_signature=False)
+                exclude_oauth_signature=False, with_realm=True)
         body_params = signature.collect_parameters(body=request.body,
                 exclude_oauth_signature=False)
         query_params = signature.collect_parameters(uri_query=request.uri_query,
@@ -465,14 +490,14 @@ class Server(object):
         params.extend(header_params)
         params.extend(body_params)
         params.extend(query_params)
-        signature_types_with_oauth_params = filter(lambda s: s[2], (
+        signature_types_with_oauth_params = list(filter(lambda s: s[2], (
             (SIGNATURE_TYPE_AUTH_HEADER, params,
                 utils.filter_oauth_params(header_params)),
             (SIGNATURE_TYPE_BODY, params,
                 utils.filter_oauth_params(body_params)),
             (SIGNATURE_TYPE_QUERY, params,
                 utils.filter_oauth_params(query_params))
-        ))
+        )))
 
         if len(signature_types_with_oauth_params) > 1:
             raise ValueError('oauth_ params must come from only 1 signature type but were found in %s' % ', '.join(
@@ -576,7 +601,6 @@ class Server(object):
         """
         raise NotImplementedError("Subclasses must implement this function.")
 
-
     def validate_requested_realm(self, client_key, realm):
         """Validates that the client may request access to the realm.
 
@@ -610,9 +634,30 @@ class Server(object):
         """
         raise NotImplementedError("Subclasses must implement this function.")
 
-    def verify_request(self, uri, http_method=u'GET', body=None,
+    def verify_request_token_request(self, uri, http_method='GET', body=None,
+            headers=None):
+        """Verify the initial request in the OAuth workflow.
+
+        During this step the client obtains a request token for use during
+        resource owner authorization (which is outside the scope of oauthlib).
+        """
+        return self.verify_request(uri, http_method=http_method, body=body,
+                headers=headers, require_resource_owner=False,
+                require_realm=True)
+
+    def verify_access_token_request(self, uri, http_method='GET', body=None,
+            headers=None):
+        """Verify the second request in the OAuth workflow.
+
+        During this step the client obtains the access token for use when
+        accessing protected resources.
+        """
+        return self.verify_request(uri, http_method=http_method, body=body,
+                headers=headers, require_verifier=True)
+
+    def verify_request(self, uri, http_method='GET', body=None,
             headers=None, require_resource_owner=True, require_verifier=False,
-            require_realm=False, required_realm=None):
+            require_realm=False, required_realm=None, require_callback=False):
         """Verifies a request ensuring that the following is true:
 
         Per `section 3.2`_ of the spec.
@@ -644,11 +689,11 @@ class Server(object):
         """
         # Only include body data from x-www-form-urlencoded requests
         headers = headers or {}
-        if (u"Content-Type" in headers and
-                headers[u"Content-Type"] == CONTENT_TYPE_FORM_URLENCODED):
+        if ("Content-Type" in headers and
+                headers["Content-Type"] == CONTENT_TYPE_FORM_URLENCODED):
             request = Request(uri, http_method, body, headers)
         else:
-            request = Request(uri, http_method, u'', headers)
+            request = Request(uri, http_method, '', headers)
 
         if self.enforce_ssl and not request.uri.lower().startswith("https://"):
             raise ValueError("Insecure transport, only HTTPS is allowed.")
@@ -661,20 +706,21 @@ class Server(object):
             raise ValueError("Duplicate OAuth entries.")
 
         oauth_params = dict(oauth_params)
-        request_signature = oauth_params.get(u'oauth_signature')
-        client_key = oauth_params.get(u'oauth_consumer_key')
-        resource_owner_key = oauth_params.get(u'oauth_token')
-        nonce = oauth_params.get(u'oauth_nonce')
-        timestamp = oauth_params.get(u'oauth_timestamp')
-        callback_uri = oauth_params.get(u'oauth_callback')
-        verifier = oauth_params.get(u'oauth_verifier')
-        signature_method = oauth_params.get(u'oauth_signature_method')
-        realm = dict(params).get(u'realm')
+        request.signature = oauth_params.get('oauth_signature')
+        request.client_key = oauth_params.get('oauth_consumer_key')
+        request.resource_owner_key = oauth_params.get('oauth_token')
+        request.nonce = oauth_params.get('oauth_nonce')
+        request.timestamp = oauth_params.get('oauth_timestamp')
+        request.callback_uri = oauth_params.get('oauth_callback')
+        request.verifier = oauth_params.get('oauth_verifier')
+        request.signature_method = oauth_params.get('oauth_signature_method')
+        request.realm = dict(params).get('realm')
 
         # The server SHOULD return a 400 (Bad Request) status code when
         # receiving a request with missing parameters.
-        if not all((request_signature, client_key, nonce,
-                    timestamp, signature_method)):
+        if not all((request.signature, request.client_key,
+                    request.nonce, request.timestamp,
+                    request.signature_method)):
             raise ValueError("Missing OAuth parameters.")
 
         # OAuth does not mandate a particular signature method, as each
@@ -685,22 +731,23 @@ class Server(object):
         # Considerations section (`Section 4`_) before deciding on which
         # method to support.
         # .. _`Section 4`: http://tools.ietf.org/html/rfc5849#section-4
-        if not signature_method in self.allowed_signature_methods:
+        if not request.signature_method in self.allowed_signature_methods:
             raise ValueError("Invalid signature method.")
 
         # Servers receiving an authenticated request MUST validate it by:
         #   If the "oauth_version" parameter is present, ensuring its value is
         #   "1.0".
-        if u'oauth_version' in oauth_params and oauth_params[u'oauth_version'] != u'1.0':
+        if ('oauth_version' in request.oauth_params and
+            request.oauth_params['oauth_version'] != '1.0'):
             raise ValueError("Invalid OAuth version.")
 
         # The timestamp value MUST be a positive integer. Unless otherwise
         # specified by the server's documentation, the timestamp is expressed
         # in the number of seconds since January 1, 1970 00:00:00 GMT.
-        if len(timestamp) != 10:
+        if len(request.timestamp) != 10:
             raise ValueError("Invalid timestamp size")
         try:
-            ts = int(timestamp)
+            ts = int(request.timestamp)
 
         except ValueError:
             raise ValueError("Timestamp must be an integer")
@@ -714,30 +761,30 @@ class Server(object):
 
         # Provider specific validation of parameters, used to enforce
         # restrictions such as character set and length.
-        if not self.check_client_key(client_key):
+        if not self.check_client_key(request.client_key):
             raise ValueError("Invalid client key.")
 
-        if not resource_owner_key and require_resource_owner:
+        if not request.resource_owner_key and require_resource_owner:
             raise ValueError("Missing resource owner.")
 
         if (require_resource_owner and not require_verifier and
-            not self.check_access_token(resource_owner_key)):
+            not self.check_access_token(request.resource_owner_key)):
             raise ValueError("Invalid resource owner key.")
 
         if (require_resource_owner and require_verifier and
-            not self.check_request_token(resource_owner_key)):
+            not self.check_request_token(request.resource_owner_key)):
             raise ValueError("Invalid resource owner key.")
 
-        if not self.check_nonce(nonce):
+        if not self.check_nonce(request.nonce):
             raise ValueError("Invalid nonce.")
 
-        if realm and not self.check_realm(realm):
+        if request.realm and not self.check_realm(request.realm):
             raise ValueError("Invalid realm. Allowed are %s" % self.realms)
 
-        if not verifier and require_verifier:
+        if not request.verifier and require_verifier:
             raise ValueError("Missing verifier.")
 
-        if require_verifier and not self.check_verifier(verifier):
+        if require_verifier and not self.check_verifier(request.verifier):
             raise ValueError("Invalid verifier.")
 
         # Servers receiving an authenticated request MUST validate it by:
@@ -751,12 +798,12 @@ class Server(object):
         # We check this before validating client and resource owner for
         # increased security and performance, both gained by doing less work.
         if require_verifier:
-            token = {"request_token": resource_owner_key}
+            token = {"request_token": request.resource_owner_key}
         else:
-            token = {"access_token": resource_owner_key}
-        if not self.validate_timestamp_and_nonce(client_key, timestamp,
-                nonce, **token):
-                return False
+            token = {"access_token": request.resource_owner_key}
+        if not self.validate_timestamp_and_nonce(request.client_key,
+                request.timestamp, request.nonce, **token):
+                return False, request
 
         # The server SHOULD return a 401 (Unauthorized) status code when
         # receiving a request with invalid client credentials.
@@ -765,12 +812,18 @@ class Server(object):
         # time request verification.
         #
         # Note that early exit would enable client enumeration
-        valid_client = self.validate_client_key(client_key)
+        valid_client = self.validate_client_key(request.client_key)
         if not valid_client:
             client_key = self.dummy_client
 
-        # Ensure a valid redirection uri is used
-        valid_redirect = self.validate_redirect_uri(client_key, callback_uri)
+        # Callback is normally never required, except for requests for
+        # a Temporary Credential as described in `Section 2.1`_
+        # .._`Section 2.1`: http://tools.ietf.org/html/rfc5849#section-2.1
+        if require_callback:
+            valid_redirect = self.validate_redirect_uri(request.client_key,
+                    request.callback_uri)
+        else:
+            valid_redirect = True
 
         # The server SHOULD return a 401 (Unauthorized) status code when
         # receiving a request with invalid or expired token.
@@ -779,15 +832,17 @@ class Server(object):
         # time request verification.
         #
         # Note that early exit would enable resource owner enumeration
-        if resource_owner_key:
+        if request.resource_owner_key:
             if require_verifier:
                 valid_resource_owner = self.validate_request_token(
-                    client_key, resource_owner_key)
+                    request.client_key, request.resource_owner_key)
+                if not valid_resource_owner:
+                    resource_owner_key = self.dummy_request_token
             else:
                 valid_resource_owner = self.validate_access_token(
-                    client_key, resource_owner_key)
-            if not valid_resource_owner:
-                resource_owner_key = self.dummy_resource_owner
+                    request.client_key, request.resource_owner_key)
+                if not valid_resource_owner:
+                    resource_owner_key = self.dummy_access_token
         else:
             valid_resource_owner = True
 
@@ -813,14 +868,16 @@ class Server(object):
         # Access to protected resources will always validate the realm but note
         # that the realm is now tied to the access token and not provided by
         # the client.
-        if ((require_realm and not resource_owner_key) or
-            (not require_resource_owner and not realm)):
-            valid_realm = self.validate_requested_realm(client_key, realm)
+        if ((require_realm and not request.resource_owner_key) or
+            (not require_resource_owner and not request.realm)):
+            valid_realm = self.validate_requested_realm(request.client_key,
+                    request.realm)
         elif require_verifier:
             valid_realm = True
         else:
-            valid_realm = self.validate_realm(client_key, resource_owner_key,
-                    uri=request.uri, required_realm=required_realm)
+            valid_realm = self.validate_realm(request.client_key,
+                    request.resource_owner_key, uri=request.uri,
+                    required_realm=required_realm)
 
         # The server MUST verify (Section 3.2) the validity of the request,
         # ensure that the resource owner has authorized the provisioning of
@@ -831,23 +888,22 @@ class Server(object):
         #
         # Note that early exit would enable resource owner authorization
         # verifier enumertion.
-        if verifier:
-            valid_verifier = self.validate_verifier(client_key,
-                resource_owner_key, verifier)
+        if request.verifier:
+            valid_verifier = self.validate_verifier(request.client_key,
+                request.resource_owner_key, request.verifier)
         else:
             valid_verifier = True
 
         # Parameters to Client depend on signature method which may vary
         # for each request. Note that HMAC-SHA1 and PLAINTEXT share parameters
 
-        request.params = filter(lambda x: x[0] != "oauth_signature", params)
-        request.signature = request_signature
+        request.params = filter(lambda x: x[0] not in ("oauth_signature", "realm"), params)
 
         # ---- RSA Signature verification ----
-        if signature_method == SIGNATURE_RSA:
+        if request.signature_method == SIGNATURE_RSA:
             # The server verifies the signature per `[RFC3447] section 8.2.2`_
             # .. _`[RFC3447] section 8.2.2`: http://tools.ietf.org/html/rfc3447#section-8.2.1
-            rsa_key = self.get_rsa_key(client_key)
+            rsa_key = self.get_rsa_key(request.client_key)
             valid_signature = signature.verify_rsa_sha1(request, rsa_key)
 
         # ---- HMAC or Plaintext Signature verification ----
@@ -857,15 +913,17 @@ class Server(object):
             #   `Section 3.4`_ and comparing it to the value received from the
             #   client via the "oauth_signature" parameter.
             # .. _`Section 3.4`: http://tools.ietf.org/html/rfc5849#section-3.4
-            client_secret = self.get_client_secret(client_key)
-            if require_verifier:
-                resource_owner_secret = self.get_request_token_secret(
-                    client_key, resource_owner_key)
-            else:
-                resource_owner_secret = self.get_access_token_secret(
-                    client_key, resource_owner_key)
+            client_secret = self.get_client_secret(request.client_key)
+            resource_owner_secret = None
+            if require_resource_owner:
+                if require_verifier:
+                    resource_owner_secret = self.get_request_token_secret(
+                        request.client_key, request.resource_owner_key)
+                else:
+                    resource_owner_secret = self.get_access_token_secret(
+                        request.client_key, request.resource_owner_key)
 
-            if signature_method == SIGNATURE_HMAC:
+            if request.signature_method == SIGNATURE_HMAC:
                 valid_signature = signature.verify_hmac_sha1(request,
                     client_secret, resource_owner_secret)
             else:
@@ -888,4 +946,4 @@ class Server(object):
             logger.info("Valid callback:\t%s" % valid_redirect)
             logger.info("Valid verifier:\t%s\t(Required: %s)" % (valid_verifier, require_verifier))
             logger.info("Valid signature:\t%s" % valid_signature)
-        return v
+        return v, request
